#database: digital_orb_db
#user: digital_orb
#password: insecure
#host: hdtv
#debug: true
#options: --single-transaction --compatible=mysql323

-- CREATE DATABASE digital_orb_db;
-- GRANT SELECT, INSERT, UPDATE, DELETE, ALTER, CREATE, DROP, INDEX ON digital_orb_db.* TO digital_orb IDENTIFIED BY 'insecure';
-- FLUSH PRIVILEGES;

-- DROP TABLE IF EXISTS tbl_tuner_class;
CREATE TABLE IF NOT EXISTS tbl_tuner_class
(
	tuner_class	VARCHAR(64) NOT NULL,
	description	TEXT,
	PRIMARY KEY pk_tbl_tuner_class(tuner_class)
);
-- DELETE FROM tbl_tuner_class;
INSERT INTO tbl_tuner_class (tuner_class, description) VALUES ('dvb', 'Linux DVB Driver (muxy)');
INSERT INTO tbl_tuner_class (tuner_class, description) VALUES ('v4l2', 'Video 4 Linux Version 2 (mencoder)');

-- DROP TABLE IF EXISTS tbl_tuner_instance;
CREATE TABLE IF NOT EXISTS tbl_tuner_instance
(
	tuner_id	INTEGER AUTO_INCREMENT NOT NULL,
	tuner_class	VARCHAR(64),
	description	TEXT,
	class_data	TEXT, -- specific data depends on the class (muxy socket path)

	PRIMARY KEY pk_tbl_tuner_instance (tuner_id)
);
-- DELETE FROM tbl_tuner_instance;
INSERT INTO tbl_tuner_instance (tuner_class, description) VALUES ('dvb', 'First Digital Card');
INSERT INTO tbl_tuner_instance (tuner_class, description) VALUES ('dvb', 'Second Digital Card');
INSERT INTO tbl_tuner_instance (tuner_class, description) VALUES ('v4l2', 'First Analog Card');

-- tbl_tuner_property (instead of tbl_tuner_details?)

-- DROP TABLE IF EXISTS tbl_channel;
CREATE TABLE IF NOT EXISTS tbl_channel
(
	channel_id	VARCHAR(64) NOT NULL, -- Even channel numbers might be '6A'
	channel_name	VARCHAR(64),
	network_name	VARCHAR(64),

	PRIMARY KEY pk_tbl_channel(channel_id)
);


-- channel_id can be whatever you want but
--   it must be unique for a 'channel'
--   a channel may be simulcast on multiple tuner_classes (ie. analog & digital but might also be cable etc)
-- a channel also has a number of other identifiers associated with it.
-- these might include xmltv names, tv guide headings etc.  Most of this is subject to change (based on geography, transmitter etc)
-- so I recommend that you use the 'number you plug into the remote' for the channel_id
-- even though the remote name might be a different identifier

-- note that 'the number you plug into the remote' seems to change a lot for DTV. This number is also
-- broadcast as part of muxy.  Given that it can change, we could either look it up at the time
-- or just put 'the number you expect' into the database and stick with that.
-- at any rate, the user will never see /this/ number so it does not matter

-- DELETE FROM tbl_channel;
INSERT INTO tbl_channel (channel_id, channel_name, network_name) VALUES ('2',  'ABC',        'ABC');
INSERT INTO tbl_channel (channel_id, channel_name, network_name) VALUES ('22', 'ABC2',       'ABC');
INSERT INTO tbl_channel (channel_id, channel_name, network_name) VALUES ('7',  'Seven',      'Seven');
INSERT INTO tbl_channel (channel_id, channel_name, network_name) VALUES ('70', 'Seven HD',   'Seven');
INSERT INTO tbl_channel (channel_id, channel_name, network_name) VALUES ('9',  'Nine',       'Nine');
INSERT INTO tbl_channel (channel_id, channel_name, network_name) VALUES ('90', 'Nine HD',    'Nine');
INSERT INTO tbl_channel (channel_id, channel_name, network_name) VALUES ('10', 'TEN',        'TEN');
INSERT INTO tbl_channel (channel_id, channel_name, network_name) VALUES ('100','TEN HD',     'TEN');
INSERT INTO tbl_channel (channel_id, channel_name, network_name) VALUES ('28', 'SBS',        'SBS');
INSERT INTO tbl_channel (channel_id, channel_name, network_name) VALUES ('29', 'SBS News',   'SBS');
INSERT INTO tbl_channel (channel_id, channel_name, network_name) VALUES ('31', 'Channel 31', 'Channel 31');

-- DROP TABLE IF EXISTS tbl_channel_tuner;
CREATE TABLE IF NOT EXISTS tbl_channel_tuner
(
	tuner_class	VARCHAR(64) NOT NULL,
	channel_id	VARCHAR(64) NOT NULL,
	available	ENUM('F', 'T') DEFAULT 'F',
	group_id	INTEGER DEFAULT 0,
	last_checked	DATETIME,

	PRIMARY KEY pk_tbl_channel_tuner (tuner_class, channel_id)
);

-- Some group_ids are allocated even though there is not currently a channel
-- to group with then.  These channels might come and go like the '7 Tennis' channel

-- channels with the same group_id should be available at the same time on a single tuner

-- a channel which is simulcast on both analog & digital will have the same channel_id

-- DELETE FROM tbl_channel_tuner;
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('dvb',  '2',  'T', 2);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('v4l2', '2',  'T', 0);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('dvb',  '22', 'T', 2);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('v4l2', '22', 'F', 0);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('dvb',  '7',  'T', 7);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('v4l2', '7',  'T', 0);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('dvb',  '70', 'T', 7);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('dvb',  '9',  'T', 9);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('v4l2', '9',  'T', 0);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('dvb',  '90', 'T', 9);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('dvb',  '10', 'T', 10);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('v4l2', '10', 'T', 0);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('dvb',  '28', 'T', 28);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('v4l2', '28', 'T', 0);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('dvb',  '29', 'T', 28);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('v4l2', '29', 'T', 0);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('dvb',  '31', 'F', 0);
INSERT INTO tbl_channel_tuner (tuner_class, channel_id, available, group_id) VALUES ('v4l2', '31', 'T', 0);


-- DROP TABLE IF EXISTS tbl_channel_identifier
CREATE TABLE IF NOT EXISTS tbl_channel_identifier
(
	tuner_class	VARCHAR(64) NOT NULL,
	channel_id	VARCHAR(64) NOT NULL,
	attribute	VARCHAR(64) NOT NULL,
	identifier	VARCHAR(256) NOT NULL,

	PRIMARY KEY pk_tbl_channel_identifier (tuner_class, channel_id, attribute)
);

-- DELETE FROM tbl_channel_identifier;
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '2',  'frequency', '226500000');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '2',  'nid',       '0x1010');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '2',  'pid',       '0x0231');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '22', 'pid',       '0x0232');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '7',  'frequency', '177500000');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '7',  'nid',       '0x1013');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '7',  'pid',       '0x0530');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '9',  'frequency', '191500000');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '9',  'nid',       '0x321d');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '9',  'pid',       '0x0430');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '10', 'frequency', '219500000');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '10', 'nid',       '0x1014');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '10', 'pid',       '0x0631');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '28', 'frequency', '536625000');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '28', 'nid',       '0x3202');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '28', 'pid',       '0x0311');
INSERT INTO tbl_channel_identifier (tuner_class, channel_id, attribute, identifier) VALUES ('dvb', '29', 'pid',       '0x0312');


CREATE TABLE IF NOT EXISTS tbl_epg_program
(
	epg_id	INTEGER AUTO_INCREMENT NOT NULL,

	channel_id	VARCHAR(64) NOT NULL,
	start_time	datetime NOT NULL,
	end_time	datetime NOT NULL,
	title		VARCHAR(256),
	sub_title	VARCHAR(256),
	description	TEXT,
	details		TEXT, -- rating, cc, ws, hd etc
	data_source	VARCHAR(64) NOT NULL,
	data_source_id	VARCHAR(64) NOT NULL,
	data_loaded	datetime NOT NULL, -- when was the last time we checked that this record is correct

	PRIMARY KEY pk_tbl_epg_program (epg_id),

	KEY k_channel_id(channel_id),
	KEY k_start_time(start_time),
	KEY k_end_time(end_time),
	KEY k_data_source(data_source),
	KEY k_data_source_id(data_source_id)
);

-- This table is used at config time to store the available EPG channel info for the web interface
-- It is not needed for day to day operation of the EPG
CREATE TABLE IF NOT EXISTS tbl_epg_channels
(
	data_source	VARCHAR(64) NOT NULL,
	channel_name	VARCHAR(64) NOT NULL,

	PRIMARY KEY pk_tbl_epg_channels(data_source, channel_name)
);


-- DVB Scheduler tables


-- does this table duplicate tbl_tuner_instance?
-- DROP TABLE IF EXISTS tbl_tuner_details;
CREATE TABLE tbl_tuner_details 
(
	tuner_id 	INT(2) NOT NULL auto_increment,
	status 		ENUM('idle', 'recording') NOT NULL DEFAULT 'idle',
	frequency 	INT(20) DEFAULT NULL,
	slots_open 	INT(4) NOT NULL DEFAULT 0,
	sock_location 	TEXT DEFAULT NULL,

	PRIMARY KEY pk_tbl_tuner_details(tuner_id)
);


-- DROP TABLE IF EXISTS tbl_schedule_details;
CREATE TABLE tbl_schedule_details 
(
	start 		DATETIME NOT NULL,
	stop 		DATETIME NOT NULL,
	channel_id	VARCHAR(64) NOT NULL,
--	stream 		VARCHAR(20) DEFAULT NULL,
	program_name 	VARCHAR(100) DEFAULT NULL,
	title		VARCHAR(256),
	rid 		INT(6) DEFAULT NULL, -- only valid while status = recording
	status		ENUM('scheduled', 'missed', 'recording', 'finished', 'canceled') NOT NULL DEFAULT 'scheduled',
--	nid 		VARCHAR(20) NOT NULL,
--	pid 		VARCHAR(20) NOT NULL,

	PRIMARY KEY pk_tbl_schedule_details (channel_id, start), -- not a good key but what?
	KEY k_program_name(program_name),
	KEY k_rid(rid),
	KEY k_status(status)
);


-- DROP TABLE IF EXISTS tbl_recording_details;
CREATE TABLE tbl_recording_details 
(
	rid 		INT(6) NOT NULL auto_increment,
	tuner_id 	INT(4) DEFAULT NULL,
	recording_slot 	INT(4) DEFAULT NULL,

	PRIMARY KEY pk_tbl_recording_details(rid)
);

-- DROP TABLE IF EXISTS tbl_auto_schedule;
CREATE TABLE tbl_auto_schedule
(
	sid		INT(6) NOT NULL auto_increment,
	rule		TEXT,
	active		ENUM('F', 'T') DEFAULT 'T',

	PRIMARY KEY pk_tbl_auto_schedule(sid)
);

-- Testing...

-- DROP TABLE IF EXISTS tbl_demo;
CREATE TABLE tbl_demo
(
	bit1		BIT(1),
	bit64		BIT(64),
	bit1n		BIT(1) NOT NULL,
	bit64n		BIT(64) NOT NULL,

	tiny1		TINYINT,
	tiny2		TINYINT UNSIGNED,
	tiny3		TINYINT(2),
	tiny4		TINYINT(2) UNSIGNED,
	tiny5		TINYINT ZEROFILL,
	tiny6		TINYINT UNSIGNED ZEROFILL,
	tiny7		TINYINT(2) ZEROFILL,
	tiny8		TINYINT(2) UNSIGNED ZEROFILL,

	tiny1n		TINYINT NOT NULL,
	tiny2n		TINYINT UNSIGNED NOT NULL,
	tiny3n		TINYINT(2) NOT NULL,
	tiny4n		TINYINT(2) UNSIGNED NOT NULL,
	tiny5n		TINYINT ZEROFILL NOT NULL,
	tiny6n		TINYINT UNSIGNED ZEROFILL NOT NULL,
	tiny7n		TINYINT(2) ZEROFILL NOT NULL,
	tiny8n		TINYINT(2) UNSIGNED ZEROFILL NOT NULL,

	bool1		BOOL,
	bool2		BOOLEAN,

	small1		SMALLINT,
	xint1		INTEGER,
	big1		BIGINT,
	small2		SMALLINT UNSIGNED,
	xint2		INTEGER UNSIGNED,
	big2		BIGINT UNSIGNED,

	small1n		SMALLINT NOT NULL,
	xint1n		INTEGER NOT NULL,
	big1n		BIGINT NOT NULL,
	small2n		SMALLINT UNSIGNED NOT NULL,
	xint2n		INTEGER UNSIGNED NOT NULL,
	big2n		BIGINT UNSIGNED NOT NULL,

	float1		FLOAT(6,2) UNSIGNED ZEROFILL,
	double1		DOUBLE(6,2) UNSIGNED ZEROFILL,
	real1		DOUBLE PRECISION(6,2) UNSIGNED ZEROFILL,
	float5		FLOAT,
	float7		FLOAT(7,2),
	float24		FLOAT(24),
	float25		FLOAT(25),

	decimal1	DECIMAL(8,2) UNSIGNED ZEROFILL,
	dec1		DEC(8,2) UNSIGNED ZEROFILL,
	numeric1	NUMERIC(8,2) UNSIGNED ZEROFILL,
	fixed1		FIXED(8,2) UNSIGNED ZEROFILL,

	decimal1n	DECIMAL(8,2) UNSIGNED ZEROFILL NOT NULL,
	dec1n		DEC(8,2) UNSIGNED ZEROFILL NOT NULL,
	numeric1n	NUMERIC(8,2) UNSIGNED ZEROFILL NOT NULL,
	fixed1n		FIXED(8,2) UNSIGNED ZEROFILL NOT NULL,

	auto		INTEGER NOT NULL AUTO_INCREMENT,

	PRIMARY KEY pk_tbl_demo(auto)


);

