#!/usr/bin/php -f
<?php
$verbose = true;

require_once "../php_lib/lib_orb.php";

load_database();

$cache_url = $config['epg_cache_url'];

function save_names()
{
	global $orb;
	global $check_names;
	global $channel_names;

	if($check_names)
	{
		$sql = "DELETE FROM tbl_epg_channels";
		$orb->execute_insert($sql);
		asort($channel_names);
		foreach($channel_names as $channel_name)
		{
			echo "$channel_name\n";
			$sql = "INSERT INTO tbl_epg_channels(data_source, channel_name) VALUES('Yahoo7', '$channel_name')";
			$orb->execute_insert($sql);
		}
	}
}

function store_program($id, $channel_name, $start, $end, $title, $subtitle, $category, $description)
{
	global $orb;
	global $verbose;

	if(!isset($orb))
	{
		return;
	}

	$sql = "SELECT channel_id FROM tbl_channel_identifier WHERE attribute='yahoo' AND identifier='$channel_name'";
	$channel = $orb->execute_count($sql);

	if($channel == "")
	{
		echo "Unknown channel $channel_name - $id, $channel_name, $start, $end, $title, $subtitle, $category, $description\n";
		return;
	}
	// we should also make sure that the channel we get is marked as available or you won't be able to
	// record this program which will cause frustration!
	$sql = "SELECT COUNT(*) FROM tbl_channel_tuner WHERE channel_id='$channel' AND available='T'";
	$available = $orb->execute_count($sql);
	if($available == 0)
	{
		if($verbose) echo "Channel $channel is not available and will be skipped\n";
		return;
	}

	// search for existing programming in this time slot
	$sql = "SELECT COUNT(*) 
                FROM   tbl_epg_program 
                WHERE  channel_id='$channel' 
                  AND (   (start_time <= '$start' AND end_time > '$start')
                       OR (start_time < '$end'   AND end_time >= '$end'  ) )";

	$conflicts = $orb->execute_count($sql);
	if($conflicts == 1)
	{
		// perhaps we already got this program...
		$sql2 = "SELECT data_source_id FROM tbl_epg_program WHERE channel_id='$channel' AND start_time = '$start' AND end_time = '$end'";
		$existing = $orb->execute_count($sql2);
		if($existing == $id)
		{
			// Yep, we already got it
			return;
		}
	}
	if($conflicts > 0)
	{
		echo "$conflicts Conflicts were found with this time slot (program id $id)\n";
		echo "+ $start    - $end    : $title\n";
		//echo $sql;
	$sql = "SELECT title, start_time, end_time 
                FROM   tbl_epg_program 
                WHERE  channel_id='$channel' 
                  AND (   (start_time <= '$start' AND end_time > '$start')
                       OR (start_time < '$end'   AND end_time >= '$end'  ) )";
		$results = $orb->execute_select($sql);
		while($row = $results->fetch_row())
		{
			echo "- $row[1] - $row[2] : $row[0]\n";
		}

		//die;
		return;
	}

	$title       = $orb->dbescape($title);
	$subtitle    = $orb->dbescape($subtitle);
	$description = $orb->dbescape($description);


	$sql = "INSERT INTO tbl_epg_program (channel_id, start_time, end_time, title, sub_title, description, details, data_source, data_source_id, data_loaded) VALUES (
		'$channel', '$start', '$end', '$title', '$subtitle', '$description', 'category=$category', 'Yahoo7', '$id', NOW())";
	echo "$sql;\n";

	$orb->execute_insert($sql);

	// flag for recording? Not here. Another process can examine all future programs and compare to the recording rules
	
}

function decode_item($item, $channel, $date, $day_time)
{
	global $orb;
	global $verbose;

	//echo $item->nodeValue;
	$h4 = $item->getElementsByTagName('h4');
	$title = $h4->item(0)->nodeValue;
	$title = fix_amp_errors($title);

	$h5 = $item->getElementsByTagName('h5');
	$subtitle = $h5->item(0)->nodeValue;

	$p = $item->getElementsByTagName('p');
	$description = $p->item(0)->nodeValue;

	$stamp = $item->getElementsByTagName('span');
	$stamp = $stamp->item(0)->nodeValue;
	$stamp = explode(' - ', $stamp);

	$start = "$date $stamp[0]";
	$start = strtotime($start);
	$start_time = date("H:i:s", $start);
	$start = date("Y-m-d H:i:s", $start);
	$end = "$date $stamp[1]";
	$end = strtotime($end);
	$end_time = date("H:i:s", $end);
	$end = date("Y-m-d H:i:s", $end);

	// check for a day wrap...
	$sql = "SELECT '$end' > '$start'";
	if($orb->execute_count($sql) == 0)
	{
		echo "Day wrap found for $title\n";
		echo "S $start_time\n";
		echo "C $day_time\n";
		echo "E $end_time\n";

		// how do we know which day this will span?
		$sql = "SELECT '$day_time' < '12:00:00'";
		echo "$sql\n";
		if($orb->execute_count($sql) == 1)
		{
			// if we loaded the morning data, let's assume this started the day before
			$alter_start = true;
		}
		else
		{
			// in the PM, lets alter the end date
			$alter_start = false;
		}

		if($alter_start)
		{
			echo "Alter start date\n";
			$sql = "SELECT DATE_SUB('$start', INTERVAL 1 DAY)";
			$start = $orb->execute_count($sql);
		}
		else
		{
			echo "Alter end date\n";
			$sql = "SELECT DATE_ADD('$end', INTERVAL 1 DAY)";
			$end = $orb->execute_count($sql);
		}

	}

	$defs = array();
	$dl = $item->getElementsByTagName('dl');
	$ds = $dl->item(0)->childNodes;
	foreach($ds as $d)
	{
		if($d->nodeName == 'dt')
		{
			$dt = $d->nodeValue;
		}
		else if($d->nodeName == 'dd')
		{
			$defs[$dt] = $d->nodeValue;
		}
	}
	//print_r($defs);
	//die;

	$id = $channel.".".strtotime($start);
	$id = strtr($id, array(' ' => '', '-' => ''));
	$category = $defs['Genre:'];
	//$href = $a->item(0)->getAttribute("href");
	if($verbose)
	{
		echo "ID: $id\n";
		echo "Title: $title\n";
		echo "Sub-title: $subtitle\n";
		echo "Start: $start\n";
		echo "End: $end\n";
		echo "Description: $description\n";
		echo "Category: $category\n";
	}
	store_program($id, $channel, $start, $end, $title, $subtitle, $category, $description);
}

function decode_tivo($href)
{
	global $orb;
	global $check_names;
	global $channel_names;

	$r = array();
	$p = strpos($href, "?");
	if($p !== false)
	{
		$href = substr($href, $p + 1);
	}
	$href = explode("&", $href);
	foreach($href as $ref)
	{
		$p = strpos($ref, '=');
		$key = substr($ref, 0, $p);
		$value = substr($ref, $p + 1);
//		$value = strtr($value, array("+" => " ", "%20" => " "));
		$value = urldecode($value);

		$r[$key] = $value;
	}

	// we need to make a unique ID
	//$id = strtr($r['pid'].'.'.$r['sid'].'.'.$r['start'].'.'.$r['cn'], array(' ' => '', '-' => ''));
	$id = strtr($r['cn'].'.'.$r['start'], array(' ' => '', '-' => ''));
	$id = strtr($id, array(' ' => '', '-' => ''));
	$channel_name = $r['cn'];
	$start = $orb->execute_count("SELECT FROM_UNIXTIME(".$r['start'].")");
	$end = $orb->execute_count("SELECT DATE_ADD('$start', INTERVAL ".$r['dur']." MINUTE)");
	$title = $r['stitle'];
	$title = fix_amp_errors($title);
	$subtitle = $r['etitle'];
	$category = '';
	$description = '';
	print_r($r);

	if($check_names)
	{
		if(!in_array($channel_name, $channel_names))
		{
			$channel_names[] = $channel_name;
		}
	}
	else
	{
		store_program($id, $channel_name, $start, $end, $title, $subtitle, $category, $description);
	}
}

function file_put_to_cache($filename, $string)
{
	global $cache_url;

	if($cache_url == "")
	{
		return;
	}

	$string = base64_encode($string);
	$url = "$cache_url?key=$filename&data=$string";
	//echo "Cache save $url\n";
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_URL, $url);
	$result = curl_exec($ch);

	if (curl_errno($ch)) 
	{
      		print curl_error($ch);
		$result = "";
	}
	else
	{
		if(curl_getinfo($ch, CURLINFO_HTTP_CODE) != 200)
		{
			$result = "";
		}
		echo "Cache save $url succeeded\n";
		echo "-$result-\n";
	}
	curl_close($ch);
}

function load_cache($filename)
{
	global $cache_url;
	global $verbose;

	if($cache_url == "")
	{
		return;
	}
	$url = "$cache_url?key=$filename";
	//echo "Cache load $url\n";
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_URL, $url);
	$result = curl_exec($ch);

	if (curl_errno($ch)) 
	{
      		print curl_error($ch);
		$result = "";
	}
	else
	{
		if(curl_getinfo($ch, CURLINFO_HTTP_CODE) != 200)
		{
			$result = "";
		}
	}
	curl_close($ch);
	if($verbose) echo "Cache load $url succeeded\n";
	echo "-$result-\n";

	return $result;
}

function extract_id($result)
{
	preg_match_all("/rel=\"\d+-\d-(\d+)\"/", $result, $id);
	return implode(",", $id[1]);
}


// Around xmas 2009, the data started to incorrectly show
// 'A & B' as 'A &B'
// Sometimes there is a real 'A&B'
// So, insert the missing space...
function fix_amp_errors($title)
{
	$title = preg_replace('/( &)([^ ])/', '\\1 \\2', $title);
	return $title;
}

	$t = time();
	$hour  = date("H", $t);
	$min   = date("i", $t);
	$date  = date("d", $t);
	$month = date("m", $t);
	$year  = date("Y", $t);
	
	$delay = 15;//120; // config item?
	$agent = "Mozilla/4.0";
	$prefix = "yahoo";
	$region = $config['epg_region'];
	if($region == "")
	{
		$region = 94; // Melbourne
	}

	// start at midnight this morning
	$days = mktime(0, 0, 0, $month, $date, $year);
	$daycount = 3;
	$check_names = false;

	if(array_key_exists(1, $_SERVER["argv"]))
	{
	        if($_SERVER["argv"][1] == "days")
	        {
	                $daycount = $_SERVER["argv"][2];
	        }
		if($_SERVER["argv"][1] == "channels")
		{
			$check_names = true;
		}
	}

	// on option to just find the channel names....?
	$channel_names = array();

	$current_offset = 0;
	$max_offset = $daycount * 24 * 3600;

	$all_ids = array();

	$times = array();

	if($check_names)
	{
		$max_offset = 0;
		echo "Reading one page of data\n";
	}
	else
	{
		echo "Reading $daycount days worth of data\n";
	}
	for($day = 0; $day < $daycount; $day++)
	{
		for($hour = 0; $hour < 24; $hour += 2)
		{
			$times[] = "0/$day/$hour";
		}
	}
//		$max_offset = 0;
/*	while(true)
	{
		$next = $days + $current_offset;
		$times[] = $next;

		$current_offset += 3 * 3600;
		if($current_offset > $max_offset)
		{
			break;
		}
	}*/

//	shuffle($times);

	foreach($times as $next)
	{
		//$url = "http://au.tv.yahoo.com/tv-guide/?hour=$hour&min=$min&date=$date&mon=$month&year=$year&tvrg=$region&next=$next";
//		$url = "http://au.tv.yahoo.com/tv-guide/$region/$months/$days/$hour";
		$url = "http://au.tv.yahoo.com/tv-guide/$region/$next/";

		//$url = "file:///home/jnewbigin/working/digital_orb/epg_internet/gem.html";

		// what is the date of this page?

		echo "$url\n"; 
//		$url = "000.html";
//		$url = "test2.html";

		$dom = new domDocument;
		$html = file_get_contents($url);
		//echo $html;
		@$dom->loadHTML($html);
		$dom->preserveWhiteSpace = false;

		$tvguide = array();
		$forms = $dom->getElementsByTagName('form');
		foreach($forms as $form)
		{
			//echo "Form: ".$form->getAttribute("id")."\n";
			if($form->getAttribute("id") == "tvguide-form")
			{
				$selects = $form->getElementsByTagName('select');
				foreach($selects as $select)
				{
					$id = $select->getAttribute("id");
					$options = $select->getElementsByTagName('option');
					foreach($options as $option)
					{
						if($option->hasAttribute("selected"))
						{
							$tvguide[$id] = $option->nodeValue;
						}
					}
				}
			}
		}
		print_r($tvguide);
		// convert the day into a date.  What happens at the end of the year...
		$day = explode(",", $tvguide['tvguide-form-day']);
		$date = date("Y-m-d", strtotime($day[1]));
		echo "The date might be $date\n";

		$day_time = $tvguide['tvguide-form-time'];
		$day_time = strtotime($day_time);
		$day_time = date("H:i:s", $day_time);
		echo "Time of day is $day_time\n";
		$lists = $dom->getElementsByTagName('li');
		foreach($lists as $list)
		{
			if(preg_match('/^row channel/', $list->getAttribute("class")))
			{
				//echo "List: ".$list->getAttribute("class")."\n";
				$h3 = $list->getElementsByTagName('h3');
				$channel = $h3->item(0)->nodeValue; // channel name
				if(!in_array($channel, $channel_names))
				{
					$channel_names[] = $channel;
				}
//if($channel != '7mate') continue;
//if($channel != 'GEM') continue;
				//echo "For channel $channel:\n";
				// list items are channels
				$items = $list->getElementsByTagName('ul');
				foreach($items as $item)
				{
					$href = "";
					$desc = "";
					//echo "- ".$item->getAttribute("class");
					//echo $item->nodeValue;
					$tivo = $item->getElementsByTagName('li');
					foreach($tivo as $t)
					{
						$class = $t->getAttribute("class");
						if($class == "tivo")
						{
							$a = $t->getElementsByTagName('a');
							$href = $a->item(0)->getAttribute("href");
							//echo "$href\n";
							// this will give us the title, subtitle?, start time, duration and channe name
//							decode_tivo($href);
						}
						else if(strpos($class, "item") !== false)
						{
							//echo 'a non-tivo item '.$t->getAttribute("class")."\n";
//							echo $t->nodeValue."\n";
							//echo "\n";
							if(!$check_names)
							{
								decode_item($t, $channel, $date, $day_time);
							}
						}
					}

					$ps = $item->getElementsByTagName('p');
					foreach($ps as $p)
					{
						if($p->getAttribute("class") == "abstract")
						{
						//echo "-".$p->getAttribute("class")."\n";
							$desc = $p->nodeValue;
						}
					}
					
					//echo "$href\n";
					//echo "$desc\n";
					//echo "=====\n";
				}
			}
			else
			{
//				echo($list->getAttribute("class")."\n");
			}
		}

		if($check_names)
		{
			break;
		}
		sleep($delay);
		continue;
/*
		$filename = "$prefix-$next.html";

		$result = load_cache($filename);
		if($result == "")
		{
			echo "Loading $url\n";
			// sleep so we don't pound the server and get kicked off
			sleep($delay);

			$ch = curl_init();
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_URL, $url);
//			curl_setopt($ch, CURLOPT_USERAGENT, $agent);
			// referer
			$result = curl_exec($ch);
	
			//if (curl_errno($ch)) 
			if(curl_getinfo($ch, CURLINFO_HTTP_CODE) != 200)
			{
		      		echo $result."\n";
			}
			else
			{
				// save the results to the cache...
				$result = extract_id($result);
				file_put_to_cache($filename, $result);
			}
			curl_close($ch);
		}
	
		$id = explode(",", $result);
		foreach($id as $i)
		{
			if(!in_array($i, $all_ids))
			{
				$all_ids[] = $i;
			}
		}*/
	}
/*
	echo "Need to load ".sizeof($all_ids)." records\n";

	// we should randomise to better co-operate with other loaders
	shuffle($all_ids);

	foreach($all_ids as $id)
	{
		$url = "http://au.tv.yahoo.com/tv-guide/broker.html?event_id=$id";

		$filename = "$prefix-id-$id.html";
		$result = load_cache($filename);
		if($result == "")
		{
			echo "Loading $url\n";
			sleep($delay);
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_URL, $url);
			curl_setopt($ch, CURLOPT_USERAGENT, $agent);
			// referer
			$result = curl_exec($ch);

			if(curl_getinfo($ch, CURLINFO_HTTP_CODE) != 200)
			{
		      		echo curl_getinfo($ch, CURLINFO_HTTP_CODE)." ".$result."\n";
			}
			else
			{
				// save the results to the cache...
				if(preg_match('/error 999/', $result))
				{
					echo "Download rejected\n";
					$result = "";
				}
				else
				{
					file_put_to_cache($filename, $result);
				}
			}
			curl_close($ch);
		}

		if($result == "")
		{
			continue;
		}

		// parse the new format data...

		// 0  = start
		// 1  = key
		// 11 = key string
		// 2  = value
		// 21 = value string
		// 22 = \ escape
		// 9  = end

		$key = "";
		$value = "";
		$state = 0; // start
		$data = array();

		for($i = 0; $i < strlen($result); $i++)
		{
			$c = $result[$i];

			if($state == 0)
			{
				if($c == "{")
				{
					$state = 1;
				}
			}
			else if($state == 1)
			{
				if($c == ':')
				{
					$state = 2;
				}
				else if($c == '"')
				{
					$state = 11;
				}
				else
				{
					$key.= $c;
				}
			}
			else if($state == 11)
			{
				if($c == '"')
				{
					$state = 1;
				}
				else
				{
					$key.= $c;
				}
			}
			else if($state == 2)
			{
				if($c == ',')
				{
					$data[$key] = $value;
					$key = "";
					$value = "";
					$state = 1;
				}
				else if($c == '}')
				{
					$data[$key] = $value;
					$state = 9;
				}
				else if($c == '"')
				{
					$state = 21;
				}
				else
				{
					$value.= $c;
				}
			}
			else if($state == 21)
			{
				if($c == '"')
				{
					$state = 2;
				}
				else if($c == "\\")
				{
					$state = 22;
				}
				else
				{
					$value.= $c;
				}
			}
			else if($state == 22)
			{
				if($c == "/")
				{
					$value.= $c;
					$state = 21;
				}
				else if($c == '"')
				{
					$value.= $c;
					$state = 21;
				}
				else
				{
					// unknown escape
					$value.= "\\$c";
					$state = 21;
				}
			}
		}
	//	print_r($data);

		$channel = $data['channel'];
		if($check_names)
		{
			if(!in_array($channel, $channel_names))
			{
				$channel_names[] = $channel;
			}
		}
		else if($channel != 'null')
		{
			$title       = $data['title'];
			$title = fix_amp_errors($title);
			$subtitle    = "";
			$category    = $data['genre'];
			$description = $data['description'];

		
			$start_date = date("Y-m-d H:i", $data['start_date']);
			$end_date   = date("Y-m-d H:i", $data['end_date']);
			store_program($id, $channel, $start_date, $end_date, $title, $subtitle, $category, $description);
		}
	}*/

	save_names();

?>
