<?php

function load_libs($libs)
{
	if(is_array($libs))
	{
		foreach($libs as $lib)
		{
			if(file_exists($lib))
			{
				include $lib;
			}
			else if(file_exists("../php_lib/$lib"))
			{
				include "../php_lib/$lib";
			}
			else if(file_exists("/opt/digital_orb/php_lib/$lib"))
			{
				include "/opt/digital_orb/php_lib/$lib";
			}
		}
	}
}

$config_loaded = false;
$config_file_name = "";
function load_config()
{
	global $config_file_name;
	global $config_loaded;
	global $config;

	if(!$config_loaded)
	{
		$files = array("digital_orb.conf", "/opt/digital_orb/config/digital_orb.conf");
		foreach($files as $filename)
		{
			if(file_exists($filename))
			{
				$config_file_name = $filename;
				$data = file($filename);
				break;
			}
		}

		if(isset($data))
		{
			foreach($data as $line)
			{
				$e = strpos($line, '=');
				if($e > 0)
				{
					$key = trim(substr($line, 0, $e));
					$value = trim(substr($line, $e + 1));
					//echo "Got '$key'='$value'\n";
					$config[$key] = $value;
				}
			}
		
			$config_loaded = true;

			@date_default_timezone_set($config['time_zone']);
		}
		else
		{
			echo "No config file found\n";
		}
	}
}

$database_loaded = false;
function load_database()
{
	global $database_loaded, $orb;
	global $config;

	if(!$database_loaded)
	{
		load_config();
	
		if(!class_exists("Database"))
		{
			load_libs(array("lib_db2.php"));
		}
	
		if(class_exists("Database"))
		{
			$orb = new Database($config['db_host'], $config['db_name'], $config['db_user'], $config['db_pass']);
			$database_loaded = $orb->connected;
		}
		else
		{
			echo "EPG Not configured\n";
			die;
		}
	}

}

$settings = array("early" => 5,
                  "late"  => 15);

function get_recording_status($id)
{
	global $orb;

	$sql = "SELECT status FROM tbl_schedule_details WHERE program_name='$id'";
	$result = $orb->execute_select($sql);
	if($row = $result->fetch())
	{
		if($row['status'] == "")
		{
			return "Scheduled";
		}
		else
		{
			return $row['status'];
		}
	}
	else
	{
		return "";
	}
}

function get_attribute_for_channel($channel, $attribute)
{
	global $orb;
	$class = 'dvb';

	$sql = "SELECT identifier 
	        FROM   tbl_channel_identifier 
		WHERE  tuner_class='$class' 
		AND    channel_id='$channel' 
		AND    attribute='$attribute'";
	return $orb->execute_count($sql);
}

function set_attribute_for_channel($channel, $attribute, $identifier)
{
	global $orb;
	$class = 'dvb';

	$sql = "REPLACE INTO tbl_channel_identifier(tuner_class, channel_id, attribute, identifier) VALUES ('$class', '$channel', '$attribute', '$identifier')";

	$orb->execute_insert($sql);
}

function get_attribute_for_channel_or_group($channel, $attribute)
{
	global $orb;

	$class = 'dvb';

	$sql = "SELECT identifier 
	        FROM   tbl_channel_identifier 
		WHERE  tuner_class='$class' 
		AND    channel_id='$channel' 
		AND    attribute='$attribute'";
	$result = $orb->execute_select($sql);
	if($row = $result->fetch_row())
	{
		$r = $row[0];
	}
	else
	{
		// look up the channel group
		$sql = "SELECT group_id 
		        FROM   tbl_channel_tuner 
			WHERE  tuner_class='$class' 
			AND    channel_id='$channel' 
			AND    available='T'";
		$gid = $orb->execute_count($sql);

		if($gid == 0)
		{
			// no data can be found.  this is a configuration error
			$r = '';
		}
		else
		{
			$sql = "SELECT channel_id 
			        FROM   tbl_channel_tuner 
				WHERE  tuner_class='$class' 
				AND    available='T' 
				AND    group_id='$gid'";
			$channels = $orb->execute_select_array($sql);
			$channels = "('".implode("', '", $channels)."')";
			$sql = "SELECT identifier 
			        FROM   tbl_channel_identifier 
				WHERE  tuner_class='$class' 
				AND    channel_id IN $channels 
				AND    attribute='$attribute'";
			$result = $orb->execute_select($sql);
			if($row = $result->fetch_row())
			{
				$r = $row[0];
			}
			else
			{
				// no attribute for this group.
				$r = '';
			}
		}
	}
	return $r;
}

function enable_record_program($id, $early = -1, $late = -1)
{
	global $orb;
	global $settings;

	if($early == -1)
	{
		$early = $settings['early'];
	}
	if($late == -1)
	{
		$late  = $settings['late'];
	}

	$sid = preg_split('/-/', $id);

	$now = $orb->execute_count("SELECT NOW()");
	$sql = "SELECT DATE_SUB(start_time, INTERVAL $early MINUTE), 
	               DATE_ADD(end_time, INTERVAL $late MINUTE), 
		       channel_id,
		       start_time < '$now' AS started,
		       end_time < '$now' AS finished,
		       title
		FROM   tbl_epg_program 
		WHERE  data_source='$sid[0]'
		AND    data_source_id='$sid[1]'";

	$result = $orb->execute_select($sql);
	if($row = $result->fetch_row())
	{
		$channel = $row[2];

		// the recording may already be scheduled or cancelled, get rid of the old record
		$sql = "DELETE FROM tbl_schedule_details WHERE program_name='$id' AND status IN ('scheduled', 'canceled')";
		$orb->execute_insert($sql);
		$title = $orb->dbescape($row[5]);
		
		$sql = "INSERT INTO tbl_schedule_details (start, stop, channel_id, program_name, title) VALUES ('$row[0]', '$row[1]', '$channel', '$id', '$title')";
		$orb->execute_insert($sql);

		return array($row[3], $row[4]);
	}
	else
	{
		return false;
	}
}

function disable_record_program($id)
{
	global $orb;

	$sid = preg_split('/-/', $id);

	//$sql = "DELETE FROM tbl_schedule_details WHERE program_name='$id' AND status = 'scheduled'";
	$sql = "UPDATE tbl_schedule_details SET status = 'canceled' WHERE program_name='$id' AND status = 'scheduled'";
	$orb->execute_insert($sql);

	$now = $orb->execute_count("SELECT NOW()");
	$sql = "SELECT start_time < '$now' AS started,
		       end_time < '$now' AS finished
		FROM   tbl_epg_program 
		WHERE  data_source='$sid[0]'
		AND    data_source_id='$sid[1]'";
	$result = $orb->execute_select($sql);
	if($row = $result->fetch_row())
	{
		return array($row[0], $row[1]);
	}
	else
	{
		return false;
	}
}

?>
